<?php

namespace App\Modules\Session;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Container\ContainerInterface;
use Slim\Views\Twig;
use App\Modules\Ldap\Ldap;
use App\Modules\FSAPI\FSAPI;
use App\Modules\FSAPI\DuplicatSessionException;
use App\Model\Session as S;
use App\Model\Call;
use App\View\Pages;
use App\Model\Exception\EmptyNumberException;
use App\Model\Exception\UserExistException;

class Session extends Pages
{

    private $ldap;
    private $rest;
    private $conn;
    private $folder;
    private $defaults;
    private $status_code;
    
    public function __construct(ContainerInterface $container)
    {
	$this->ldap = $container->get('settings')['ldap'];
	$this->rest = $container->get('settings')['rest'];
	$this->folder = $container->get('settings')['media']['folder'];
	$this->defaults = $container->get('settings')['defaults'];
	$this->conn = new Ldap($this->ldap);
	$this->status_code = $container->get('settings')['call_status'];
	parent::__construct();
    }

    private function active_session(){
	$api = new FSAPI($this->rest);
	$sessions = $api->session();
	return $sessions;
    }
    
    private function session_info($uuid){
	$api = new FSAPI($this->rest);
	$session = $api->session_info($uuid);
	return $session;
    }

    private function session_update($uuid, $data){
	$api = new FSAPI($this->rest);
	$response = $api->session_update($uuid, $data);
	return $response;
    }
    
    public function status(ServerRequestInterface $request,
			   ResponseInterface $response, array $args): ResponseInterface
    {
	$params = $request->getQueryParams();
	$current_session = $this->active_session();

	$temp_files =  glob("$this->folder/*.wav");
	$files=[];
	/* echo var_dump($temp_files); */
	foreach ($temp_files as $f){
	    $files[] = $f;
	};
	$media = [];
	foreach ($files as $file){
	    $media[] = basename($file);
	}
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$temp_users = [];

	if(isset($params['state'])){
	    if($params['state'] == 'active' ){
		if($session->status == "active") {
		    return $this->render($request, $response, 'session/active_calls.html', [
			'session' => $session,
			'status_code' => $this->status_code
		    ]);
		} else {
		    return $response->withStatus(301);
		}

	    }
	} else {
	    return $this->render($request, $response, 'session/main.html', [
		'session' => $session,
		'media' => $media,
		'current_session' => $current_session,
		'currentPath' => $_SERVER['REQUEST_URI'],
		'status_code' => $this->status_code
	    ]);
	}
    }

    public function config(ServerRequestInterface $request,
			   ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$temp_files =  glob("$this->folder/*.wav");
	$files=[];
	foreach ($temp_files as $f){
	    $files[] = $f;
	};
	$media = [''];
	foreach ($files as $file){
	    $media[] = basename($file);
	}
	$uuid = $args['uuid'];
	/* echo var_dump($name); */
	$session = $this->session_info($uuid);
	return $this->render($request, $response, 'session/config.html', [
	    'session' => $session,
	    'media' => $media,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI'],
	    'status_code' => $this->status_code
	]);
    }

    public function update_config(ServerRequestInterface $request,
				  ResponseInterface $response, array $args): ResponseInterface
    {
	$uuid = $args['uuid'];
	$params = $request->getParsedBody();
	$newname = explode(' ', $params['name']);
	$newname = implode('_', $newname);
	$data['name'] = $newname;
	$data['media'] = $this->folder."/".$params['media'];
	$data['anonce'] = $this->folder."/".$params['anonce'];
	$data['parallel'] = $params['parallel'];
	$data['repeate'] = $params['repeate'];
	$data['pause'] = $params['pause'];

	$resp = $this->session_update($uuid, $data);
	$_SESSION['message'][] = ["настройки успешно применены", 'info'];
	/* echo var_dump($resp); */
	return $this->redirect($response, "/ui/session/".$uuid);
    }

    public function dict(ServerRequestInterface $request,
			 ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$groups = $this->conn->search_departs("*");

	$params = $request->getQueryParams();
	if (isset($params['query']) AND $params['query']!=''){
	    $query = $params['query'];
	} else {
	    $query = '*';
	}
	return $this->render($request, $response, 'session/dict.html', [
	    'session' => $session,
	    'departs' => $groups,
	    'query' => $query,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI'],
	    'status_code' => $this->status_code
	]);
    }

    public function depart(ServerRequestInterface $request,
			   ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$depart_name = $args['depart'];
	$depart = $this->conn->get_depart($depart_name);

	return $this->render($request, $response, 'session/depart.html', [
	    'session' => $session,
	    'depart' => $depart,
	    'depart_name' => $depart_name,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI'],
	    'status_code' => $this->status_code
	]);
    }

    public function search(ServerRequestInterface $request,
			   ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$params = $request->getQueryParams();
	if (isset($params['query'])) {
	    if($params['query']!='') {
		$query = $params['query'];
	    } else {
		$query = '*';
	    }
	    $info = $this->conn->search($query);
	    $groups = $this->conn->search_departs($query);
	    if ($info == [] AND $groups != []){
		foreach($groups as $g){
		    [$g['name'] => $val] = $this->conn->get_depart($g['name']);
		    $info[$g['name']] = $val;
		}
	    }

	    return $this->render($request, $response, 'session/search.html', [
		'session' => $session,
		'search' => $info,
		'query' => $query,
		'current_session' => $current_session,
		'currentPath' => $_SERVER['REQUEST_URI'],
		'status_code' => $this->status_code
	    ]);
	} else {
	    return $this->render($request, $response, 'session/search_form.html', [
		'session' => $session,
	    ]);
	}
    }


    public function groups(ServerRequestInterface $request,
			   ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$groups = $this->conn->search_groups('*');

	return $this->render($request, $response, 'session/groups.html', [
	    'session' => $session,
	    'groups' => $groups,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI'],
	    'status_code' => $this->status_code
	]);
    }

    public function group(ServerRequestInterface $request,
			  ResponseInterface $response, array $args): ResponseInterface
    {
	$current_session = $this->active_session();
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$group_name = $args['group'];
	$group = $this->conn->get_group($group_name);

	return $this->render($request, $response, 'session/group.html', [
	    'session' => $session,
	    'group' => $group,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI'],
	    'status_code' => $this->status_code
	]);
    }

    public function subscriber(ServerRequestInterface $request,
			       ResponseInterface $response, array $args): ResponseInterface
    {
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$params = $request->getParsedBody();
	if(isset($params['add']))
	{
	    $subscriber_name = $params['add'];

	    try
	    {
		$this->add_subscriber($session, $subscriber_name);
		/* $_SESSION['message'][] = [$subscriber_name." успешно добавлен",
		   'info']; */
	    }
	    catch(EmptyNumberException $ex){
		/* $_SESSION['message'][] = [$ex->getMessage()." отсутствует номер",
		   'error']; */
	    }
	    catch(UserExistException $ex){
		/* $_SESSION['message'][] = [$ex->getMessage()." уже добавлен",
		   'error']; */
	    }
	} elseif(isset($params['remove']))
	{
	    $subscriber_name = $params['remove'];
	    $this->remove_subscriber($session, $subscriber_name);
	    /* $_SESSION['message'][] = [$subscriber_name." успешно удален",
	       'info']; */
	}
	/* echo var_dump($currentPath); */
	$from = $request->getHeader('Referer');
	return $this->redirect($response, $from);
	/* return $this->render($request, $response, 'session/group.html', [
	   'session' => $session,
	   'currentPath' => $_SERVER['REQUEST_URI'],
	   'status_code' => $this->status_code
	   ]); */
    }

    public function add_group(ServerRequestInterface $request,
			      ResponseInterface $response, array $args): ResponseInterface
    {
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$params = $request->getParsedBody();
	if(isset($params['add']))
	{
	    $group_name = $params['add'];
	    $group = $this->conn->get_group($group_name);
	    $users = $group[$group_name];
	    foreach($users as $user){
		try
		{
		    $this->add_subscriber($session, $user['cn'][0]);
		    /* $_SESSION['message'][] = [$user['cn'][0]." успешно добавлен",
		       'info']; */
		}
		catch(EmptyNumberException $ex){
		    /* $_SESSION['message'][] = [$ex->getMessage()." отсутствует номер",
		       'error']; */
		}
		catch(UserExistException $ex){
		    /* $_SESSION['message'][] = [$ex->getMessage()." уже добавлен",
		       'error']; */
		}
	    }
	} elseif(isset($params['remove']))
	{
	    $group_name = $params['remove'];
	    $group = $this->conn->get_group($group_name);
	    $users = $group[$group_name];
	    foreach($users as $user){
		$this->remove_subscriber($session, $user['cn'][0]);
		/* $_SESSION['message'][] = [$user['cn'][0]." успешно удален",
		   'info']; */
	    }
	}
	/* echo var_dump($currentPath); */
	$from = $request->getHeader('Referer');
	return $this->redirect($response, $from);
	/* return $this->render($request, $response, 'session/group.html', [
	   'session' => $session,
	   'currentPath' => $_SERVER['REQUEST_URI'],
	   'status_code' => $this->status_code
	   ]); */
    }

    public function add_depart(ServerRequestInterface $request,
			       ResponseInterface $response, array $args): ResponseInterface
    {
	$uuid = $args['uuid'];
	$session = $this->session_info($uuid);
	$params = $request->getParsedBody();
	if(isset($params['add']))
	{
	    $depart_name = $params['add'];
	    $depart = $this->conn->get_depart($depart_name);
	    $users = $depart[$depart_name];
	    foreach($users as $user){
		try
		{
		    $this->add_subscriber($session, $user['cn'][0]);
		    /* $_SESSION['message'][] = [$user['cn'][0]." успешно добавлен",
		       'info']; */
		}
		catch(EmptyNumberException $ex){
		    /* $_SESSION['message'][] = [$ex->getMessage()." отсутствует номер",
		       'error']; */
		}
		catch(UserExistException $ex){
		    /* $_SESSION['message'][] = [$ex->getMessage()." уже добавлен",
		       'error']; */
		}
	    }
	} elseif(isset($params['remove']))
	{
	    $depart_name = $params['remove'];
	    $depart = $this->conn->get_depart($depart_name);
	    $users = $depart[$depart_name];
	    foreach($users as $user){
		$this->remove_subscriber($session, $user['cn'][0]);
		/* $_SESSION['message'][] = [$user['cn'][0]." успешно удален",
		   'info']; */
	    }
	}
	/* echo var_dump($currentPath); */
	$from = $request->getHeader('Referer');
	return $this->redirect($response, $from);
	/* return $this->render($request, $response, 'session/group.html', [
	   'session' => $session,
	   'currentPath' => $_SERVER['REQUEST_URI'],
	   'status_code' => $this->status_code
	   ]); */
    }
    
    
    private function add_subscriber($session, $name){
	$subscriber = $this->conn->get_user($name);
	$depart = $this->conn->search_depart_by_member($name);
	/* $user['name' => $subscriber['cn'][0],
	   'name' => $subscriber['cn'][0],
	   ] */
	if(isset($subscriber['w'])){
	    $weidth = $subscriber['w'];
	} else {
	    $weidth = 999;
	}
	$user = [$subscriber['cn'][0],
		 $subscriber['employeetype'][0],
		 $subscriber['mobile'][0],
		 $depart[0]['name'],
		 $weidth];
	$session->add_subscriber($user);
	$resp = $this->session_update($session->uuid,
				      ['numbers' => $session->numbers,
				       'meta' => $session->meta]);
    }

    private function remove_subscriber($session, $name){
	$session->remove_subscriber($name);
	$resp = $this->session_update($session->uuid,
				      ['numbers' => $session->numbers,
				       'meta' => $session->meta]);
    }
    

    public function get(ServerRequestInterface $request,
			ResponseInterface $response, array $args): ResponseInterface
    {
	/* session_start();
	   $view = Twig::fromRequest($request); */
	$users = [];
	/* echo var_dump($_SESSION['select']); */
	if(isset($_SESSION['select'])){
	    foreach ($_SESSION['select'] as $user => $val){
		$users[] = $user;
	    }
	}
	$data = $this->conn->get_users($users);
	$session = [];
	foreach ($data as $u){
	    $session[$_SESSION['select'][$u['cn'][0]]][] = $u;
	}
	/* echo var_dump($session); */
	$api = new FSAPI($this->rest);
	$current_session = $this->active_session();
	/* echo var_dump($current_session); */
	return $this->render($request, $response, 'session.html', [
	    'data' => $data,
	    'groups' => [],
	    'session' => $session,
	    'current_session' => $current_session,
	    'currentPath' => $_SERVER['REQUEST_URI']
	]);
    }

    
    public function new(ServerRequestInterface $request,
			ResponseInterface $response, array $args): ResponseInterface
    {
	session_start();
	$view = Twig::fromRequest($request);
	$params = $request->getParsedBody();

	$session_name = $params['session_name'];
	$session_name = explode(' ', $session_name);
	$session_name = implode('_', $session_name);
	
	$session['name'] = $session_name;
	/* $data['media'] = $this->folder."/".$params['media']; */
	/* $data['numbers'] = $params['number']; */
	/* $session['numbers'] = $numbers; */
	/* $data['parallel'] = $params['parallel'];
	   $data['repeate'] = $params['repeate']; */
	/* $session['meta'] = $list; */
	/* echo var_dump($session['numbers']); */

	$filename = $this->defaults['phrase'];
	
	if(($fp = fopen($filename, 'r')) !== FALSE){
	    while (($tmp = fgetcsv($fp, 100, ",")) !== FALSE) {
		$actions[] = [$tmp[0] => $tmp[1]];
	    }
	} else {
	    $actions[] = [['play' => 'anonce'],
			  ['sleep' => '1000'],
			  ['play' => 'message']];
	};

	$session['actions'] = $actions;
	$session['from_user'] = $this->defaults['number_from'];
	$session['repeate'] = $this->defaults['repeate'];
	$session['parallel'] = $this->defaults['parallel'];
	$session['pause'] = $this->defaults['wait'];
	$session['media'] = $this->folder."/"."message.wav";
	$session['anonce'] = $this->folder."/"."anonce.wav";
	
	$api = new FSAPI($this->rest);
	/* echo var_dump($api); */

	/* echo var_dump(json_encode($users)); */
	try
	{
	    $session = $api->create_session($session);
	    unset($_SESSION['select']);
	    return $response->withStatus(302)->
	    withHeader('Location', "/ui/session/". $session['uuid']);
	}
	catch(DuplicatSessionException $ex){
	    $_SESSION['message'][] = [$ex->getMessage(),
				      'error'];
	    return $response->withStatus(302)->
	    withHeader('Location', "/ui/session");
	}

    }

    public function start(ServerRequestInterface $request,
			  ResponseInterface $response, array $args): ResponseInterface
    {
	$uuid = $args['uuid'];
	$params = $request->getParsedBody();
	$api = new FSAPI($this->rest);
	if (isset($params['start'])){
	    $current_session = $api->start_session($uuid);
	    return $this->redirect($response, "/ui/session/$uuid");
	} elseif(isset($params['close'])){
	    $current_session = $api->stop_session($uuid);
	    return $response->withStatus(302)->withHeader('Location', "/ui/session");
	} elseif(isset($params['stop'])){
	    $current_session = $api->stop_session($uuid);
	    return $response->withStatus(302)->withHeader('Location', "/ui/session/$uuid");
	} elseif(isset($params['restart'])){
	    /* echo var_dump($params['numbers']); */
	    $data['numbers'] = $params['numbers'];
	    $current_session = $api->restart_session($uuid, $data);
	    return $response->withStatus(302)->withHeader('Location', "/ui/session/$uuid");
	}
	elseif(isset($params['report'])){
	    return $response->withStatus(302)->withHeader('Location', "/ui/report/$uuid");
	}
	elseif(isset($params['archive'])){
	    return $response->withStatus(302)->withHeader('Location', "/ui/archive");
	}
    }

    public function test_call(ServerRequestInterface $request,
			      ResponseInterface $response, array $args): ResponseInterface
    {
	$params = $request->getQueryParams();
	$media = $params['media'];
	$anonce = $params['anonce'];
	/* $media = "message.wav"; */
	$api = new FSAPI($this->rest);
	$filename = $this->defaults['phrase'];
	$number = $this->defaults['number_record'];
	
	if(($fp = fopen($filename, 'r')) !== FALSE){
	    while (($tmp = fgetcsv($fp, 100, ",")) !== FALSE) {
		$actions[] = [$tmp[0] => $tmp[1]];
	    }	} else {
		$actions[] = [['play' => 'anonce'],
			      ['sleep' => '1000'],
			      ['play' => 'message']];
	    };

	$data['actions'] = $actions;
	$data['from_user'] = $this->defaults['number_from'];
	$data['number'] = $number;
	$data['name'] = $media;
	$data['anonce'] = $anonce;
	$api->test_call($data);
	/* echo var_dump($data); */
	$response->getBody()->write("start test call");
	return $response;
    }
}
