<?php

namespace App\Modules\User;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Container\ContainerInterface;
use Slim\Views\Twig;
use App\Modules\Ldap\Ldap;
use App\View\Pages;

class Operator extends Pages
{
    private $ldap;
    private $conn;
    
    public function __construct(ContainerInterface $container)
    {
        $this->ldap = $container->get('settings')['ldap'];
        $this->conn = new Ldap($this->ldap);
    }

    public function list(ServerRequestInterface $request,
                         ResponseInterface $response, array $args): ResponseInterface
    {
        $view = Twig::fromRequest($request);
        $params = $request->getQueryParams();
       	$operators = $this->conn->operator_list();
        // echo var_dump($operators);
        if (isset($params['query']) AND $params['query']!=''){
            $query = $params['query'];
            $info = $this->conn->search($query);
            // $info = $this->conn->search_with_group_without_list($query, $name);
            $groups = $this->conn->search_departs($query);
            
            foreach($groups as $g) {
                $temp = $this->conn->get_depart($g['name']);
                $info[$g['name']] = $temp[$g['name']];
                // echo var_dump($info);
            }
            return $this->render($request, $response, 'operator.html', [
                'operators' => $operators,
                'data' => $info,
                'groups' => $groups,
                'query' => $query
            ]);
        } else {
            
            return $this->render($request, $response, 'operator.html', [
		'operators' => $operators
            ]);
        }
    }

    public function delete(ServerRequestInterface $request,
                           ResponseInterface $response, array $args): ResponseInterface
    {
        $name = $args['name'];
        $operators = $this->conn->del_operator($name);
	return $response->withStatus(302)
			->withHeader('Location', "/ui/operator");
    }

    public function edit(ServerRequestInterface $request,
                         ResponseInterface $response, array $args): ResponseInterface
    {
        $name = $args['name'];
        $view = Twig::fromRequest($request);
        $operator = $this->conn->get_operator($name);
        // echo var_dump($operator);
        return $this->render($request, $response, 'edit_operator.html', [
            'operator' => $operator
        ]);
    }

    public function create(ServerRequestInterface $request,
                           ResponseInterface $response, array $args): ResponseInterface
    {
        $data = $request->getParsedBody();
        $name = $args['name'];
        $operator['login'] = $data['login'];
        $operator['role'] = $data['role'];
        $operator['password'] = $data['password'];
        
        $operator = $this->conn->edit_operator($name, $operator);
        echo var_dump($operator);
        return $response->withStatus(302)
			->withHeader('Location', "/ui/operator");
    }
}
